# -*- coding: utf-8 -*-

import select
import time
import heapq
import logging
import threading
from collections import defaultdict, deque
from .utils import is_int

''' You can only use instance(). Don't create a Loop() '''

_ioloop_lock = threading.RLock()
_ioloop_cls = None
_ioloop = None


def instance():
    global _ioloop_cls, _ioloop
    if _ioloop is not None:
        return _ioloop

    with _ioloop_lock:
        if _ioloop is not None:
            return _ioloop
        else:
            if 'epoll' in select.__dict__:
                from .impl import epoll_loop
                logging.debug('using epoll')
                _ioloop_cls = epoll_loop.EpollLoop
            elif 'kqueue' in select.__dict__:
                from .impl import kqueue_loop
                logging.debug('using kqueue')
                _ioloop_cls = kqueue_loop.KqueueLoop
            else:
                from .impl import select_loop
                logging.debug('using select')
                _ioloop_cls = select_loop.SelectLoop

            _ioloop = _ioloop_cls()
            return _ioloop


def current():
    return _ioloop


# these values are defined as the same as poll
MODE_NULL = 0x00
MODE_IN = 0x01
MODE_OUT = 0x04
MODE_ERR = 0x08
MODE_HUP = 0x10
MODE_NVAL = 0x20


class TimeoutHandler(object):
    def __init__(self, callback, deadline, args, kwargs):
        '''deadline here is absolute timestamp'''
        self.callback = callback
        self.deadline = deadline
        self.args=args
        self.kwargs=kwargs
        self.canceled = False

    def __cmp__(self, other):
        return (self.deadline > other.deadline) - (self.deadline < other.deadline)

    def __eq__(self, other):
        return self.deadline == other.deadline

    def __gt__(self, other):
        return self.deadline > other.deadline

    def __lt__(self, other):
        return self.deadline < other.deadline

    def __ge__(self, other):
        return self.deadline >= other.deadline

    def __le__(self, other):
        return self.deadline <= other.deadline

    def __ne__(self, other):
        return self.deadline != other.deadline

    def __call__(self):
        self.callback(*self.args, **self.kwargs)


class IOLoop(object):
    def __init__(self):
        self._handlers = deque()
        self._run_handlers = deque()
        self._timeout_handlers = []
        self._fd_handlers = defaultdict(list)
        self._stopped = False

    def _poll(self, timeout):
        raise NotImplementedError()

    def _add_fd(self, fd, mode):
        raise NotImplementedError()

    def _remove_fd(self, fd):
        raise NotImplementedError()

    def _modify_fd(self, fd, mode):
        raise NotImplementedError()

    def add_fd(self, fd, mode, callback):
        handlers = self._fd_handlers[fd]
        new_handlers = []
        if not handlers:
            new_handlers.append((callback, fd, mode))
            self._add_fd(fd, mode)
        else:
            new_mode = MODE_NULL
            for hcallback, hfd, hmode in handlers:
                if hcallback != callback:
                    new_mode |= hmode
                    new_handlers.append((hcallback, hfd, hmode))
            new_handlers.append((callback, fd, mode))
            new_mode |= mode
            self._modify_fd(fd, new_mode)
        self._fd_handlers[fd] = new_handlers
        return True

    def update_fd(self, fd, mode, callback):
        handlers = self._fd_handlers[fd]
        if not handlers:
            return False
        new_handlers = []
        new_mode = MODE_NULL
        for hcallback, hfd, hmode in handlers:
            if hcallback == callback:
                new_mode |= mode
                new_handlers.append((hcallback, hfd, mode))
            else:
                new_mode |= hmode
                new_handlers.append((hcallback, hfd, hmode))
        self._modify_fd(fd, new_mode)
        self._fd_handlers[fd] = new_handlers
        return True

    def remove_fd(self, fd, callback):
        handlers = self._fd_handlers[fd]
        if not handlers:
            return False
        if len(handlers) == 1:
            if handlers[0][0] == callback:
                self._remove_fd(fd)
                del self._fd_handlers[fd]
        else:
            new_handlers = []
            new_mode = MODE_NULL
            for hcallback, hfd, hmode in handlers:
                if hcallback != callback:
                    new_mode |= hmode
                    new_handlers.append((hcallback, hfd, hmode))
            self._modify_fd(fd, new_mode)
            self._fd_handlers[fd] = new_handlers
        return True

    def clear_fd(self, fd):
        if fd in self._fd_handlers:
            if self._fd_handlers[fd]:
                del self._fd_handlers[fd]
                self._remove_fd(fd)
                return True
            del self._fd_handlers[fd]
        return False

    def start(self):
        while not self._stopped:
            timeout = 1
            if self._timeout_handlers:
                cur_time = time.time()
                while self._timeout_handlers:
                    handler = self._timeout_handlers[0]
                    if handler.canceled:
                        heapq.heappop(self._timeout_handlers)
                    elif handler.deadline <= cur_time:
                        heapq.heappop(self._timeout_handlers)
                        try:
                            handler.callback(*handler.args, **handler.kwargs)
                        except Exception as e:
                            logging.exception("loop callback timeout error:%s", e)
                    else:
                        timeout = self._timeout_handlers[0].deadline - cur_time
                        if timeout > 1:
                            timeout = 1
                        break

            if self._handlers:
                timeout = 0
                
            fds_ready = self._poll(timeout)
            for fd, mode in fds_ready:
                handlers = self._fd_handlers[fd]
                for hcallback, hfd, hmode in handlers:
                    if hmode & mode != 0:
                        try:
                            hcallback()
                        except Exception as e:
                            logging.exception("loop callback error:%s", e)

            # call handlers without fd
            self._handlers, self._run_handlers = self._run_handlers, self._handlers
            while self._run_handlers:
                callback, args, kwargs = self._run_handlers.popleft()
                try:
                    callback(*args, **kwargs)
                except Exception  as e:
                    logging.exception("loop callback error:%s", e)

    def stop(self):
        self._stopped = True

    def add_async(self, callback, *args, **kwargs):
        self._handlers.append((callback, args, kwargs))

    def add_timeout(self, timeout, callback, *args, **kwargs):
        handler = TimeoutHandler(callback, time.time() + timeout, args, kwargs)
        heapq.heappush(self._timeout_handlers, handler)
        return handler

    def cancel_timeout(self, handler):
        if handler.__class__ == TimeoutHandler:
            handler.canceled = True
        else:
            self._timeout_handlers.remove(handler)


# -*- coding: utf-8 -*-

import os
import select
from collections import defaultdict
from ..loop import IOLoop, MODE_NULL, MODE_IN, MODE_OUT
from ..utils import is_py3

try:
    MAX_EVENTS = int(os.environ.get("SEVENT_KQUEUE_MAX_EVENTS", 1024))
except:
    MAX_EVENTS = 1024

class KqueueLoop(IOLoop):
    def __init__(self):
        super(KqueueLoop, self).__init__()
        self._kqueue = select.kqueue()
        self._fds = {}

    def _control(self, fd, mode, flags):
        events = []
        if mode & MODE_IN:
            events.append(select.kevent(fd, select.KQ_FILTER_READ, flags))
        if mode & MODE_OUT:
            events.append(select.kevent(fd, select.KQ_FILTER_WRITE, flags))
        for e in events:
            self._kqueue.control([e], 0)

    def _poll(self, timeout):
        if timeout < 0:
            timeout = None  # kqueue behaviour
        events = self._kqueue.control(None, MAX_EVENTS, timeout)
        results = defaultdict(lambda: MODE_NULL)
        for e in events:
            fd = e.ident
            if e.filter == select.KQ_FILTER_READ:
                results[fd] |= MODE_IN
            elif e.filter == select.KQ_FILTER_WRITE:
                results[fd] |= MODE_OUT
        return results.items() if is_py3 else results.iteritems()

    def _add_fd(self, fd, mode):
        self._fds[fd] = mode
        self._control(fd, mode, select.KQ_EV_ADD)

    def _remove_fd(self, fd):
        self._control(fd, self._fds[fd], select.KQ_EV_DELETE)
        del self._fds[fd]

    def _modify_fd(self, fd, mode):
        self._remove_fd(fd)
        self._add_fd(fd, mode)



# -*- coding: utf-8 -*-
# 15/8/6
# create by: snower

import os
import time
import socket
import dnslib
from collections import defaultdict
from .loop import instance
from .event import EventEmitter
from .utils import ensure_bytes, is_py3

QTYPE_ANY = 255
QTYPE_A = 1
QTYPE_AAAA = 28
QTYPE_CNAME = 5
QTYPE_NS = 2
QCLASS_IN = 1

STATUS_OPENED = 0
STATUS_CLOSED = 1

class DNSCache(object):
    def __init__(self, loop, default_ttl = 60):
        self._loop = loop or instance()
        self.default_ttl = default_ttl
        self._cache = defaultdict(list)
        self._last_resolve_time = time.time()

    def append(self, hostname, rrs):
        rrcs = []
        now = time.time()
        for rrc in self._cache[hostname]:
            if rrc.ttl_expried_time <= now:
                continue
            rrcs.append(rrc)
        self._cache[hostname] = rrcs

        for rr in rrs:
            has_cache = False
            for rrc in self._cache[hostname]:
                if rr == rrc:
                    has_cache = True
                    break
            if has_cache:
                continue
            setattr(rr, "ttl_expried_time", time.time() + (rr.ttl or self.default_ttl))
            self._cache[hostname].append(rr)

        if self._last_resolve_time - now >= 120:
            self._loop.add_async(self.resolve)
            self._last_resolve_time = now

    def get(self, hostname):
        if not self._cache[hostname]:
            return None, hostname

        now = time.time()
        if self._last_resolve_time - now >= 120:
            self._loop.add_async(self.resolve)
            self._last_resolve_time = now

        for rrc in self._cache[hostname]:
            if rrc.ttl_expried_time <= now:
                continue
            return str(rrc.rdata), hostname
        return None, hostname

    def remove(self, hostname):
        if hostname in self._cache:
            self._cache.pop(hostname)

        now = time.time()
        if self._last_resolve_time - now >= 120:
            self._loop.add_async(self.resolve)
            self._last_resolve_time = now

    def resolve(self):
        now = time.time()
        epried_hostnames = []
        for hostname, rrs in self._cache.items():
            rrcs = []
            for rr in rrs:
                if rr.ttl_expried_time <= now:
                    continue
                rrcs.append(rr)
            if rrcs:
                self._cache[hostname] = rrcs
            else:
                epried_hostnames.append(hostname)

        for hostname in epried_hostnames:
            self.remove(hostname)

    def clear(self):
        self._cache = defaultdict(list)

    def __getitem__(self, hostname):
        return self.get(hostname)[0]

    def __delitem__(self, hostname):
        return self.remove(hostname)

    def __contains__(self, hostname):
        return bool(self.get(hostname)[0])

class DNSResolver(EventEmitter):
    _instance = None

    @classmethod
    def default(cls):
        if cls._instance is None:
            cls._instance = cls()
        return cls._instance

    def __init__(self, loop=None, servers=None, hosts=None, resolve_timeout = None, resend_timeout = 0.5):
        super(DNSResolver, self).__init__()

        self._loop = loop or instance()
        self._servers = []
        self._server6s = []
        self._hosts = hosts or {}

        self._cache = DNSCache(self._loop)
        self._queue = defaultdict(list)
        self._loading = defaultdict(int)
        self._socket = None
        self._socket6 = None
        self._status = STATUS_OPENED

        if not servers:
            servers = self.parse_resolv()
        for server in servers:
            inet_type = self.is_ip(server)
            if inet_type == socket.AF_INET:
                self._servers.append(server)
            elif inet_type == socket.AF_INET6:
                self._server6s.append(server)

        if not hosts:
            self.parse_hosts()

        self._resolve_timeout = resolve_timeout if resolve_timeout else ((len(self._servers) + len(self._server6s)) * resend_timeout + 4)
        self._resend_timeout = resend_timeout

    def on_resolve(self, callback):
        self.on("resolve", callback)

    def once_resolve(self, callback):
        self.once("resolve", callback)

    def create_socket(self):
        from .udp import Socket, Socket6
        self._socket = Socket(self._loop)
        self._socket.on("data", self.on_data)
        self._socket.on("close", self.on_close)

    def create_socket6(self):
        from .udp import Socket6
        self._socket6 = Socket6(self._loop)
        self._socket6.on("data", self.on_data)
        self._socket6.on("close", self.on_close)

    def parse_resolv(self):
        servers = []
        try:
            with open('/etc/resolv.conf', 'rb') as f:
                content = f.readlines()
                for line in content:
                    line = line.strip()
                    if line:
                        if line.startswith(b'nameserver'):
                            parts = line.split()
                            if len(parts) >= 2:
                                server = parts[1]
                                if self.is_ip(server):
                                    if is_py3 and type(server) != str:
                                        server = server.decode('utf8')
                                    servers.append(server)
        except IOError:
            pass
        if not servers:
            try:
                servers = str(os.environ.get("SEVENT_NAMESERVER", '')).split(",")
            except:
                servers = ['8.8.4.4', '8.8.8.8']
        return servers

    def parse_hosts(self):
        etc_path = '/etc/hosts'
        if 'WINDIR' in os.environ:
            etc_path = os.environ['WINDIR'] + '/system32/drivers/etc/hosts'
        try:
            with open(etc_path, 'rb') as f:
                for line in f.readlines():
                    line = line.strip()
                    parts = line.split()
                    if len(parts) >= 2:
                        ip = parts[0]
                        if self.is_ip(ip):
                            for i in range(1, len(parts)):
                                hostname = parts[i]
                                if hostname:
                                    self._hosts[hostname] = ip
        except IOError:
            self._hosts[b'localhost'] = '127.0.0.1'

    def call_callback(self, hostname, ip):
        if not self._queue[hostname]:
            return
        callbacks = self._queue.pop(hostname)
        hostname = hostname.decode("utf-8") if is_py3 and type(hostname) != str else hostname
        for callback in callbacks:
            self._loop.add_async(callback, hostname, ip)

        self._loop.add_async(self.emit_resolve, self, hostname, ip)

    def on_data(self, socket, buffer):
        while buffer:
            data, address = buffer.next()
            try:
                answer = dnslib.DNSRecord.parse(data)
                hostname = b".".join(answer.q.qname.label)
                rrs = [rr for rr in answer.rr if rr.rtype == answer.q.qtype]
                self._loading[hostname] -= 1

                if rrs:
                    self._cache.append(hostname, rrs)
                    self.call_callback(hostname, str(rrs[0].rdata))
                elif self._loading[hostname] <= 0:
                    self.call_callback(hostname, None)

                if self._loading[hostname] <= 0:
                    self._loading.pop(hostname)
            except Exception as e:
                pass

    def send_req(self, hostname, server_index = 0):
        if not self._servers:
            return

        question = dnslib.DNSRecord.question(hostname, 'A')
        if self._socket is None:
            self.create_socket()
        self._socket.write((bytes(question.pack()), (self._servers[server_index], 53)))
        self._loading[hostname] += 1

        def on_timeout():
            if server_index + 1 >= len(self._servers):
                return
            if hostname not in self._cache:
                self.send_req(hostname, server_index + 1)
        self._loop.add_timeout(self._resend_timeout, on_timeout)

    def send_req6(self, hostname, server_index = 0):
        if not self._server6s:
            return

        question = dnslib.DNSRecord.question(hostname, 'AAAA')
        if self._socket6 is None:
            self.create_socket6()
        self._socket6.write((bytes(question.pack()), (self._server6s[server_index], 53)))
        self._loading[hostname] += 1

        def on_timeout():
            if server_index + 1 >= len(self._server6s):
                return
            if hostname not in self._cache:
                self.send_req6(hostname, server_index + 1)
        self._loop.add_timeout(self._resend_timeout, on_timeout)

    def resolve(self, hostname, callback, timeout = None):
        if self._status == STATUS_CLOSED:
            return callback(hostname.decode("utf-8") if is_py3 and type(hostname) != str else hostname, None)

        hostname = ensure_bytes(hostname)
        if not hostname:
            callback(hostname.decode("utf-8") if is_py3 and type(hostname) != str else hostname, None)

        elif self.is_ip(hostname):
            callback(hostname.decode("utf-8") if is_py3 and type(hostname) != str else hostname, hostname)
        elif hostname in self._hosts:
            callback(hostname.decode("utf-8") if is_py3 and type(hostname) != str else hostname, self._hosts[hostname])
        elif hostname in self._cache:
            callback(hostname.decode("utf-8") if is_py3 and type(hostname) != str else hostname, self._cache[hostname])
        else:
            try:
                if not self._queue[hostname]:
                    self._queue[hostname].append(callback)
                    self.send_req(hostname)
                    self.send_req6(hostname)

                    def on_timeout():
                        if hostname not in self._cache:
                            self.call_callback(hostname, None)
                    self._loop.add_timeout(timeout or self._resolve_timeout, on_timeout)
                else:
                    self._queue[hostname].append(callback)
            except Exception as e:
                self.call_callback(hostname, None)

    def flush(self):
        self._cache.clear()

    def on_close(self, socket):
        if self._status == STATUS_CLOSED:
            return

        if self._socket == socket:
            self._socket = None
        else:
            self._socket6 = None

    def close(self):
        if self._status == STATUS_CLOSED:
            return

        self._status = STATUS_CLOSED
        if self._socket:
            self._socket.close()
        if self._socket6:
            self._socket6.close()

        for hostname, callbacks in self._queue:
            for callback in callbacks:
                callback(hostname, None)

    def inet_pton(self, family, addr):
        if family == socket.AF_INET:
            return socket.inet_aton(addr)
        elif family == socket.AF_INET6:
            if '.' in addr:  # a v4 addr
                v4addr = addr[addr.rindex(':') + 1:]
                v4addr = socket.inet_aton(v4addr)
                if is_py3:
                    v4addr = list(map(lambda x: ('%02X' % x), ensure_bytes(v4addr)))
                else:
                    v4addr = map(lambda x: ('%02X' % ord(x)), ensure_bytes(v4addr))
                v4addr.insert(2, ':')
                newaddr = addr[:addr.rindex(':') + 1] + ''.join(v4addr)
                return self.inet_pton(family, newaddr)
            dbyts = [0] * 8  # 8 groups
            grps = addr.split(':')
            for i, v in enumerate(grps):
                if v:
                    dbyts[i] = int(v, 16)
                else:
                    for j, w in enumerate(grps[::-1]):
                        if w:
                            dbyts[7 - j] = int(w, 16)
                        else:
                            break
                    break
            if is_py3:
                return b''.join([(chr(i // 256) + chr(i % 256)).encode("utf-8") for i in dbyts])
            return b''.join([(chr(i // 256) + chr(i % 256)) for i in dbyts])
        else:
            raise RuntimeError("What family?")

    def is_ip(self, address):
        for family in (socket.AF_INET, socket.AF_INET6):
            try:
                if is_py3 and type(address) != str:
                    address = address.decode('utf8')
                self.inet_pton(family, address)
                return family
            except (TypeError, ValueError, OSError, IOError):
                pass
        return False



# -*- coding: utf-8 -*-
# 2014/12/28
# create by: snower

import logging
import socket
import errno
from .event import EventEmitter
from .loop import instance, MODE_IN, MODE_OUT
from .dns import DNSResolver
from .buffer import Buffer, cbuffer, RECV_BUFFER_SIZE

STATE_INITIALIZED = 0x01
STATE_CONNECTING = 0x02
STATE_STREAMING = 0x04
STATE_BINDING = 0x08
STATE_CLOSING = 0x10
STATE_CLOSED = 0x20

class Socket(EventEmitter):
    MAX_BUFFER_SIZE = None
    RECV_BUFFER_SIZE = RECV_BUFFER_SIZE

    @classmethod
    def config(cls, max_buffer_size=None, recv_buffer_size=RECV_BUFFER_SIZE, **kwargs):
        cls.MAX_BUFFER_SIZE = max_buffer_size
        cls.RECV_BUFFER_SIZE = recv_buffer_size

    def __init__(self, loop=None, dns_resolver=None, max_buffer_size = None):
        super(Socket, self).__init__()
        self._loop = loop or instance()
        self._dns_resolver = dns_resolver

        self._socket = None
        self._fileno = 0
        self._socket_family = 2
        self._read_handler = None
        self._write_handler = None
        self._has_drain_event = False

        self._max_buffer_size = max_buffer_size or self.MAX_BUFFER_SIZE
        self._rbuffers = Buffer(max_buffer_size = self._max_buffer_size)
        self._wbuffers = None
        self._address_cache = {}
        self._state = STATE_INITIALIZED

    def _connect(self, address, callback):
        def resolve_callback(hostname, ip):
            if not ip:
                return self._loop.add_async(self._error, Exception('can not resolve hostname %s' % str(address)))

            try:
                addrinfo = socket.getaddrinfo(ip, address[1], 0, 0, socket.SOL_UDP)
                if not addrinfo:
                    return self._loop.add_async(self._error, Exception('can not resolve hostname %s' % str(address)))

                addr = addrinfo[0]
                self._socket = socket.socket(addr[0], addr[1], addr[2])
                self._fileno = self._socket.fileno()
                self._socket_family = addr[0]
                self._socket.setblocking(False)
                self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                self._read_handler = self._loop.add_fd(self._fileno, MODE_IN, self._read_cb)
                self._write_handler = None
                callback((ip, address[1]))
            except socket.error as e:
                if e.args[0] not in (errno.EINPROGRESS, errno.EWOULDBLOCK):
                    return self._loop.add_async(self._error, Exception("connect error %s %s" % (str(address), e)))
            except Exception as e:
                return self._loop.add_async(self._error, e)

        self._state = STATE_CONNECTING
        if not self._dns_resolver:
            self._dns_resolver = DNSResolver.default()
        self._dns_resolver.resolve(address[0], resolve_callback)

    def __del__(self):
        self.close()

    def on(self, event_name, callback):
        super(Socket, self).on(event_name, callback)

        if event_name == "drain":
            self._has_drain_event = True

    def once(self, event_name, callback):
        super(Socket, self).once(event_name, callback)

        if event_name == "drain":
            self._has_drain_event = True

    def remove_listener(self, event_name, callback):
        super(Socket, self).remove_listener(event_name, callback)

        if not self._events[event_name] and not self._events_once[event_name]:
            if event_name == "drain":
                self._has_drain_event = False

    def on_data(self, callback):
        self.on("data", callback)

    def on_close(self, callback):
        self.on("close", callback)

    def on_error(self, callback):
        self.on("error", callback)

    def on_drain(self, callback):
        self.on("drain", callback)

    def once_data(self, callback):
        self.once("data", callback)

    def once_close(self, callback):
        self.once("close", callback)

    def once_error(self, callback):
        self.once("error", callback)

    def once_drain(self, callback):
        self.once("drain", callback)

    def end(self):
        if self._state in (STATE_STREAMING, STATE_BINDING):
            if not self._write_handler:
                self.close()
            else:
                self._state = STATE_CLOSING

    def close(self):
        if self._state == STATE_CLOSED:
            return
        if self._read_handler:
            try:
                self._loop.remove_fd(self._fileno, self._read_cb)
            except Exception as e:
                logging.error("socket close remove_fd error:%s", e)
            self._read_handler = False
        if self._write_handler:
            try:
                self._loop.remove_fd(self._fileno, self._write_cb)
            except Exception as e:
                logging.error("socket close remove_fd error:%s", e)
            self._write_handler = False

        self._state = STATE_CLOSED
        def on_close():
            if self._socket:
                try:
                    self._loop.clear_fd(self._fileno)
                except Exception as e:
                    logging.error("server close clear_fd error: %s", e)
                try:
                    self._socket.close()
                except Exception as e:
                    logging.error("socket close socket error:%s", e)

            try:
                self.emit_close(self)
            except Exception as e:
                logging.exception("tcp emit close error:%s", e)
            self.remove_all_listeners()
            self._rbuffers = None
            self._wbuffers = None
            self._address_cache = None
        self._loop.add_async(on_close)

    def _error(self, error):
        self._loop.add_async(self.emit_error, self, error)
        self._loop.add_async(self.close)
        logging.error("socket error: %s", error)

    def drain(self):
        if self._state in (STATE_STREAMING, STATE_BINDING):
            if self._read_handler:
                try:
                    self._loop.remove_fd(self._fileno, self._read_cb)
                except Exception as e:
                    return self._error(e)
                self._read_handler = False

    def regain(self):
        if self._state in (STATE_STREAMING, STATE_BINDING):
            if not self._read_handler:
                try:
                    self._read_handler = self._loop.add_fd(self._fileno, MODE_IN, self._read_cb)
                except Exception as e:
                    self._error(e)

    def _read_cb(self):
        if self._state in (STATE_STREAMING, STATE_BINDING):
            if self._read():
                self._loop.add_async(self.emit_data, self, self._rbuffers)

    if cbuffer is None:
        def _read(self):
            last_data_len = self._rbuffers._len
            while self._read_handler:
                try:
                    data, address = self._socket.recvfrom(self.RECV_BUFFER_SIZE)
                    self._rbuffers.write(data, address)
                except socket.error as e:
                    if e.args[0] in (errno.EWOULDBLOCK, errno.EAGAIN):
                        break
                    else:
                        self._error(e)
                        if self._rbuffers._len > self._rbuffers._drain_size and not self._rbuffers._full:
                            self._rbuffers.do_drain()
                        return False
                except Exception as e:
                    self._error(e)
                    return

            if last_data_len < self._rbuffers._len:
                if self._rbuffers._len > self._rbuffers._drain_size and not self._rbuffers._full:
                    self._rbuffers.do_drain()
                return True
            return False
    else:
        def _read(self):
            try:
                r = self._rbuffers.socket_recvfrom(self._fileno, self._socket_family)
            except Exception as e:
                self._error(e)
                if self._rbuffers._len > self._rbuffers._drain_size and not self._rbuffers._full:
                    self._rbuffers.do_drain()
                return False

            if self._rbuffers._len > self._rbuffers._drain_size and not self._rbuffers._full:
                self._rbuffers.do_drain()
            return r

    def _write_cb(self):
        if self._state != STATE_CLOSED:
            if self._write():
                if self._has_drain_event:
                    self._loop.add_async(self.emit_drain, self)
                if self._write_handler:
                    try:
                        self._loop.remove_fd(self._fileno, self._write_cb)
                    except Exception as e:
                        return self._error(e)
                    self._write_handler = False

                if self._state == STATE_CLOSING:
                    self.close()

    if cbuffer is None:
        def _write(self):
            while self._wbuffers:
                data = self._wbuffers
                address = self._wbuffers._buffer_odata
                try:
                    if data._buffer_index > 0:
                        r = self._socket.sendto(memoryview(data._buffer)[data._buffer_index:], address)
                    else:
                        r = self._socket.sendto(data._buffer, address)
                    data._buffer_index += r
                    data._len -= r

                    if data._buffer_index >= data._buffer_len:
                        if data._len > 0:
                            data._buffer = data._buffers.popleft()
                            data._buffer_odata = data._buffers_odata.popleft()
                            data._buffer_index, data._buffer_len = 0, len(data._buffer)
                        else:
                            data._buffer_index, data._buffer_len, data._buffer, data._buffer_odata = 0, 0, b'', None
                            if data._full and data._len < data._regain_size:
                                data.do_regain()
                        continue
                    else:
                        if data._full and data._len < data._regain_size:
                            data.do_regain()
                        return False
                except socket.error as e:
                    if e.args[0] not in (errno.EWOULDBLOCK, errno.EAGAIN):
                        self._error(e)
                    if data._full and data._len < data._regain_size:
                        data.do_regain()
                    return False
                except Exception as e:
                    self._error(e)
                    return
            return True
    else:
        def _write(self):
            try:
                self._wbuffers.socket_sendto(self._fileno, self._socket_family)
                if self._wbuffers:
                    if self._wbuffers._full and self._wbuffers._len < self._wbuffers._regain_size:
                        self._wbuffers.do_regain()
                    return False
            except Exception as e:
                self._error(e)
                if self._wbuffers._full and self._wbuffers._len < self._wbuffers._regain_size:
                    self._wbuffers.do_regain()
                return False

            if self._wbuffers._full and self._wbuffers._len < self._wbuffers._regain_size:
                self._wbuffers.do_regain()
            return True

    def write(self, data):
        def do_write():
            if self._state not in (STATE_STREAMING, STATE_BINDING):
                if self._state == STATE_INITIALIZED:
                    _, address = self._wbuffers[0]
                    self._connect(address, lambda address: do_write())
                    return False

                if not self._socket:
                    return False
                self._state = STATE_STREAMING

            if not self._write_handler:
                if self._write():
                    if self._has_drain_event:
                        self._loop.add_async(self.emit_drain, self)
                    return True
                try:
                    self._write_handler = self._loop.add_fd(self._fileno, MODE_OUT, self._write_cb)
                except Exception as e:
                    self._error(e)
            return False

        if data.__class__ == Buffer:
            if self._wbuffers != data:
                if not self._wbuffers:
                    self._wbuffers = data
                else:
                    while data:
                        self._wbuffers.write(*data.next())
            return do_write()
        else:
            if self._wbuffers is None:
                self._wbuffers = Buffer(max_buffer_size = self._max_buffer_size)

        data, address = data
        if address[0] not in self._address_cache:
            def resolve_callback(hostname, ip):
                if not ip:
                    return self._error(Exception('can not resolve hostname %s' % str(address)))
                self._address_cache[hostname] = ip
                self._wbuffers.write(data, (ip, address[1]))
                return do_write()

            if not self._dns_resolver:
                self._dns_resolver = DNSResolver.default()
            self._dns_resolver.resolve(address[0], resolve_callback)
            return False
        else:
            ip = self._address_cache[address[0]]
            self._wbuffers.write(data, (ip, address[1]))
            return do_write()

class Socket6(Socket):
    pass

class Server(Socket):
    def __init__(self, loop=None):
        super(Server, self).__init__(loop)

    def bind(self, address):
        def do_bind(address):
            try:
                self._socket.bind(address)
                self._state = STATE_BINDING
            except Exception as e:
                self._loop.add_async(self._error, e)

        self._connect(address, do_bind)

    def __del__(self):
        self.close()

class Server6(Server):
    pass



# -*- coding: utf-8 -*-
# 15/1/27
# create by: snower

import os
import time
import logging
from collections import deque
from .event import EventEmitter
from .loop import current
from .utils import ensure_unicode, is_py3

try:
    RECV_BUFFER_SIZE = int(os.environ.get("SEVENT_RECV_BUFFER_SIZE", 0))
except:
    RECV_BUFFER_SIZE = 0

try:
    MAX_BUFFER_SIZE = int(os.environ.get("SEVENT_MAX_BUFFER_SIZE", 4 * 1024 * 1024))
except:
    MAX_BUFFER_SIZE = 4 * 1024 * 1024

try:
    if not os.environ.get("SEVENT_NOUSE_CBUFFER", False):
        from . import cbuffer
        BaseBuffer = cbuffer.Buffer
        if RECV_BUFFER_SIZE:
            try:
                cbuffer.socket_set_recv_size(RECV_BUFFER_SIZE)
            except: pass
    else:
        cbuffer = None
except ImportError:
    logging.warning("cbuffer is not supported")
    cbuffer = None

RECV_BUFFER_SIZE = RECV_BUFFER_SIZE or 8 * 1024 - 64

if cbuffer is None:
    class BaseBuffer(object):
        def __init__(self):
            self._buffer = b''
            self._buffer_odata = None
            self._buffer_len = 0
            self._buffers = deque()
            self._buffers_odata = deque()
            self._len = 0
            self._buffer_index = 0

        def join(self):
            if self._buffer_index > 0 or self._buffer_len - self._buffer_index < self._len:
                if self._buffer_index < self._buffer_len:
                    self._buffers.appendleft(self._buffer[self._buffer_index:])
                if len(self._buffers) > 1:
                    data = b"".join(self._buffers)
                    self._buffers.clear()
                else:
                    data = self._buffers.popleft()
                if self._buffers_odata:
                    self._buffer_odata = self._buffers_odata[-1]
                    self._buffers_odata.clear()
                self._buffer = data
                self._buffer_len = len(data)
                self._buffer_index = 0
            return self._buffer

        def write(self, data, odata = None):
            if self._buffer_len > 0:
                self._buffers.append(data)
                self._buffers_odata.append(odata)
                self._len += len(data)
            else:
                self._buffer = data
                self._buffer_odata = odata
                self._buffer_len = len(data)
                self._len += self._buffer_len
            return self

        def read(self, size=-1):
            if size < 0:
                if self._buffer_len - self._buffer_index < self._len:
                    self.join()
                    buffer = self._buffer
                elif self._buffer_index > 0:
                    buffer = self._buffer[self._buffer_index:]
                else:
                    buffer = self._buffer
                buffer_odata = self._buffer_odata
                self._buffer_index, self._buffer_len, self._buffer, self._buffer_odata, self._len = 0, 0, b'', None, 0
                return (buffer, buffer_odata) if buffer_odata else buffer

            if self._len < size:
                return b""

            if self._buffer_len - self._buffer_index < size:
                self.join()
                data = self._buffer[:size]
                self._buffer_index = size
            else:
                data = self._buffer[self._buffer_index: self._buffer_index + size]
                self._buffer_index += size
            buffer_odata = self._buffer_odata
            self._len -= size

            if self._buffer_index >= self._buffer_len:
                if self._len > 0:
                    self._buffer = self._buffers.popleft()
                    self._buffer_odata = self._buffers_odata.popleft()
                    self._buffer_index, self._buffer_len = 0, len(self._buffer)
                else:
                    self._buffer_index, self._buffer_len, self._buffer, self._buffer_odata = 0, 0, b'', None
            return (data, buffer_odata) if buffer_odata else data

        def next(self):
            if self._buffer_index > 0:
                data = self._buffer[self._buffer_index:]
                self._len -= self._buffer_len - self._buffer_index
            else:
                data = self._buffer
                self._len -= self._buffer_len
            buffer_odata = self._buffer_odata

            if self._len > 0:
                self._buffer = self._buffers.popleft()
                self._buffer_odata = self._buffers_odata.popleft()
                self._buffer_index, self._buffer_len = 0, len(self._buffer)
            else:
                self._buffer_index, self._buffer_len, self._buffer, self._buffer_odata = 0, 0, b'', None
            return (data, buffer_odata) if buffer_odata else data

        def __len__(self):
            return self._len

        def __str__(self):
            buffer = self.join()

            if is_py3:
                return ensure_unicode(buffer)
            return (buffer, self._buffer_odata) if self._buffer_odata else buffer

        def __nonzero__(self):
            return self._len > 0

        def __getitem__(self, index):
            buffer = self.join()
            return (buffer.__getitem__(index), self._buffer_odata) if self._buffer_odata else buffer.__getitem__(index)

        def __hash__(self):
            buffer = self.join()
            return buffer.__hash__()

class Buffer(EventEmitter, BaseBuffer):
    def __init__(self, max_buffer_size = None):
        EventEmitter.__init__(self)
        BaseBuffer.__init__(self)

        self._loop = current()
        self._full = False
        self._drain_size = max_buffer_size or MAX_BUFFER_SIZE
        self._regain_size = self._drain_size * 0.5
        self._drain_time = time.time()
        self._regain_time = time.time()

    def on_drain(self, callback):
        self.on("drain", callback)

    def on_regain(self, callback):
        self.on("regain", callback)

    def once_drain(self, callback):
        self.once("drain", callback)

    def once_regain(self, callback):
        self.once("regain", callback)

    def do_drain(self):
        self._full = True
        self._drain_time = time.time()
        try:
            self.emit_drain(self)
        except Exception as e:
            logging.exception("buffer emit drain error:%s", e)

    def do_regain(self):
        self._full = False
        self._regain_time = time.time()
        try:
            self.emit_regain(self)
        except Exception as e:
            logging.exception("buffer emit regain error:%s", e)

    def write(self, data, odata = None):
        if odata is None:
            BaseBuffer.write(self, data)
        else:
            BaseBuffer.write(self, data, odata)

        if self._len > self._drain_size and not self._full:
            self.do_drain()
        return self

    def read(self, size=-1):
        data = BaseBuffer.read(self, size)

        if self._full and self._len < self._regain_size:
            self.do_regain()
        return data

    def next(self):
        data = BaseBuffer.next(self)

        if self._full and self._len < self._regain_size:
            self.do_regain()
        return data

    def __iter__(self):
        while True:
            data = self.next()
            if not data:
                raise StopIteration()
            yield data

    def __contains__(self, item):
        buffer = self.join()

        return buffer.__contains__(item)



# -*- coding: utf-8 -*-

import logging
from collections import defaultdict

def null_emit_callback(*args, **kwargs):
    return None

class EventEmitter(object):
    def __init__(self):
        self._events = defaultdict(set)
        self._events_once = defaultdict(set)

    def on(self, event_name, callback):
        self._events[event_name].add(callback)

        if len(self._events[event_name]) == 1 and not self._events_once[event_name]:
            setattr(self, "emit_" + event_name, callback)
        else:
            setattr(self, "emit_" + event_name, self.emit_callback(event_name))

    def once(self, event_name, callback):
        self._events_once[event_name].add(callback)

        if not self._events[event_name] and len(self._events_once[event_name]) == 1:
            def emit_callback(*args, **kwargs):
                try:
                    return callback(*args, **kwargs)
                finally:
                    setattr(self, "emit_" + event_name, null_emit_callback)
                    self._events_once[event_name] = set()
            setattr(self, "emit_" + event_name, emit_callback)
        else:
            setattr(self, "emit_" + event_name, self.emit_callback(event_name))

    def remove_listener(self, event_name, callback):
        try:
            self._events[event_name].remove(callback)
        except KeyError:
            pass

        try:
            self._events_once[event_name].remove(callback)
        except KeyError:
            pass

        if len(self._events[event_name]) == 1 and not self._events_once[event_name]:
            setattr(self, "emit_" + event_name, list(self._events[event_name])[0])
        elif not self._events[event_name] and len(self._events_once[event_name]) == 1:
            callback = list(self._events_once[event_name])[0]
            def emit_callback(*args, **kwargs):
                try:
                    return callback(*args, **kwargs)
                finally:
                    setattr(self, "emit_" + event_name, null_emit_callback)
                    self._events_once[event_name] = set()
            setattr(self, "emit_" + event_name, emit_callback)
        elif self._events[event_name] and self._events_once[event_name]:
            setattr(self, "emit_" + event_name, self.emit_callback(event_name))
        else:
            setattr(self, "emit_" + event_name, null_emit_callback)

    def remove_all_listeners(self, event_name=None):
        if event_name is None:
            for event_name in set(list(self._events.keys()) + list(self._events_once.keys())):
                setattr(self, "emit_" + event_name, null_emit_callback)
            self._events = defaultdict(set)
            self._events_once = defaultdict(set)
        else:
            setattr(self, "emit_" + event_name, null_emit_callback)
            self._events[event_name] = set()
            self._events_once[event_name] = set()

    def emit_callback(self, event_name):
        def _(*args, **kwargs):
            for cb in self._events[event_name]:
                try:
                    cb(*args, **kwargs)
                except Exception as e:
                    logging.exception('error when calling callback:%s',e)

            callbacks = self._events_once[event_name]
            if callbacks:
                self._events_once[event_name] = set()
                while callbacks:
                    cb = callbacks.pop()
                    try:
                        cb(*args, **kwargs)
                    except Exception as e:
                        logging.exception('error when calling callback:%s',e)
        return _

    def emit(self, event_name, *args, **kwargs):
        for cb in self._events[event_name]:
            try:
                cb(*args, **kwargs)
            except Exception as e:
                logging.exception('error when calling callback:%s', e)

        callbacks = self._events_once[event_name]
        if callbacks:
            self._events_once[event_name] = set()
            while callbacks:
                cb = callbacks.pop()
                try:
                    cb(*args, **kwargs)
                except Exception as e:
                    logging.exception('error when calling callback:%s', e)

    def __getattr__(self, item):
        if item[:5] == "emit_":
            event_name = item[5:]
            if len(self._events[event_name]) == 1 and not self._events_once[event_name]:
                callback = list(self._events[event_name])[0]
                setattr(self, "emit_" + event_name, callback)
                return callback

            if not self._events[event_name] and len(self._events_once[event_name]) == 1:
                callback = list(self._events_once[event_name])[0]

                def emit_callback(*args, **kwargs):
                    try:
                        return callback(*args, **kwargs)
                    finally:
                        setattr(self, "emit_" + event_name, null_emit_callback)
                        self._events_once[event_name] = set()

                setattr(self, "emit_" + event_name, emit_callback)
                return emit_callback

            if self._events[event_name] and self._events_once[event_name]:
                callback = self.emit_callback(event_name)
                setattr(self, "emit_" + event_name, callback)
                return callback

            setattr(self, "emit_" + event_name, null_emit_callback)
            return null_emit_callback

        elif item[:3] == "on_":
            return lambda *args, **kwargs: self.on(item[3:], *args, **kwargs)
        return object.__getattribute__(self, item)


# -*- coding: utf-8 -*-

version = '0.1.0'
version_info = (0,1,0)

from .loop import instance, current
from .event import EventEmitter
from . import tcp
from . import udp
from .buffer import Buffer
from .dns import DNSResolver



#!/usr/bin/env python

import os
import sys
import platform
from setuptools import setup, Extension

if platform.system() != 'Windows' and platform.python_implementation() == "CPython":
    ext_modules = [Extension('sevent/cbuffer', sources=['sevent/cbuffer.c'])]
else:
    ext_modules = []

if os.path.exists("README.md"):
    if sys.version_info[0] >= 3:
        with open("README.md", encoding="utf-8") as fp:
            long_description = fp.read()
    else:
        with open("README.md") as fp:
            long_description = fp.read()
else:
    long_description = ''

setup(
    name = 'sevent',
    version = '0.1.0',
    packages = ['sevent', 'sevent.impl'],
    ext_modules = ext_modules,
    package_data = {
        '': ['README.md'],
    },
    install_requires = [
        'dnslib>=0.9.7',
    ],
    author = 'snower',
    author_email = 'sujian199@gmail.com',
    url = 'https://github.com/snower/sevent',
    license = 'MIT',
    description = 'lightweight event loop',
    long_description = long_description,
    long_description_content_type="text/markdown",
)



# -*- coding: utf-8 -*-
# 18/5/25
# create by: snower

import sys

if sys.version_info[0] >= 3:
    is_py3 = True
    unicode_type = str
    byte_type = bytes

    def is_int(v):
        return v.__class__ == int

    iter_range = range
else:
    is_py3 = False
    unicode_type = unicode
    byte_type = str


    def is_int(v):
        return v.__class__ == int or v.__class__ == long

    iter_range = xrange

def ensure_bytes(s):
    if isinstance(s, unicode_type):
        return s.encode("utf-8")
    return s

def ensure_unicode(s):
    if isinstance(s, byte_type):
        return s.decode("utf-8")
    return s


# -*- coding: utf-8 -*-

import logging
import socket
import errno
from . import event
from .loop import instance, MODE_IN, MODE_OUT
from .buffer import Buffer, cbuffer, RECV_BUFFER_SIZE
from .dns import DNSResolver

MSG_FASTOPEN = 0x20000000

STATE_INITIALIZED = 0x01
STATE_CONNECTING = 0x02
STATE_STREAMING = 0x04
STATE_LISTENING = 0x08
STATE_CLOSING = 0x10
STATE_CLOSED = 0x20

class Socket(event.EventEmitter):
    MAX_BUFFER_SIZE = None
    RECV_BUFFER_SIZE = RECV_BUFFER_SIZE

    @classmethod
    def config(cls, max_buffer_size = None, recv_buffer_size = RECV_BUFFER_SIZE, **kwargs):
        cls.MAX_BUFFER_SIZE = max_buffer_size
        cls.RECV_BUFFER_SIZE = recv_buffer_size

    def __init__(self, loop=None, socket=None, address=None, dns_resolver = None, max_buffer_size = None):
        super(Socket, self).__init__()
        self._loop = loop or instance()
        self._socket = socket
        self._fileno = 0
        self._socket_family = 2
        self._address = address
        self._dns_resolver = dns_resolver or DNSResolver.default()
        self._connect_handler = False
        self._connect_timeout = 5
        self._connect_timeout_handler = None
        self._read_handler = False
        self._write_handler = False
        self._max_buffer_size = max_buffer_size or self.MAX_BUFFER_SIZE
        self._rbuffers = Buffer(max_buffer_size= self._max_buffer_size)
        self._wbuffers = None
        self._state = STATE_INITIALIZED
        self._is_enable_fast_open = False
        self._is_enable_nodelay = False
        self._is_resolve = False
        self._has_drain_event = False

        if self._socket:
            self._state = STATE_STREAMING
            try:
                self._fileno = self._socket.fileno()
                self._socket.setblocking(False)
                self._read_handler = self._loop.add_fd(self._fileno, MODE_IN, self._read_cb)
            except Exception as e:
                self._loop.add_async(self._error, e)

    @property
    def address(self):
        return self._address

    @property
    def socket(self):
        return self._socket

    def __del__(self):
        self.close()

    def on(self, event_name, callback):
        super(Socket, self).on(event_name, callback)

        if event_name == "drain":
            self._has_drain_event = True

    def once(self, event_name, callback):
        super(Socket, self).once(event_name, callback)

        if event_name == "drain":
            self._has_drain_event = True

    def remove_listener(self, event_name, callback):
        super(Socket, self).remove_listener(event_name, callback)

        if not self._events[event_name] and not self._events_once[event_name]:
            if event_name == "drain":
                self._has_drain_event = False

    def on_connect(self, callback):
        self.on("connect", callback)

    def on_data(self, callback):
        self.on("data", callback)

    def on_end(self, callback):
        self.on("end", callback)

    def on_close(self, callback):
        self.on("close", callback)

    def on_error(self, callback):
        self.on("error", callback)

    def on_drain(self, callback):
        self.on("drain", callback)

    def once_connect(self, callback):
        self.once("connect", callback)

    def once_data(self, callback):
        self.once("data", callback)

    def once_end(self, callback):
        self.once("end", callback)

    def once_close(self, callback):
        self.once("close", callback)

    def once_error(self, callback):
        self.once("error", callback)

    def once_drain(self, callback):
        self.once("drain", callback)

    def enable_fast_open(self):
        self._is_enable_fast_open = True

    @property
    def is_enable_fast_open(self):
        return self._is_enable_fast_open

    def enable_nodelay(self):
        if self._socket:
            try:
                self._socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
            except Exception as e:
                logging.warning('nodela error: %s', e)
                self._is_enable_nodelay = False
                return
        self._is_enable_nodelay = True

    @property
    def is_enable_nodelay(self):
        return self._is_enable_nodelay

    def end(self):
        if self._state not in (STATE_INITIALIZED, STATE_CONNECTING, STATE_STREAMING):
            return

        if self._state in (STATE_INITIALIZED, STATE_CONNECTING):
            self.close()
        else:
            if self._write_handler:
                self._state = STATE_CLOSING
            else:
                self.close()

    def close(self):
        if self._state == STATE_CLOSED:
            return

        if self._state == STATE_CONNECTING and self._connect_handler:
            try:
                self._loop.remove_fd(self._fileno, self._connect_cb)
            except Exception as e:
                logging.error("socket close remove_fd error:%s", e)
            self._connect_handler = False
        elif self._state in (STATE_STREAMING, STATE_CLOSING):
            if self._read_handler:
                try:
                    self._loop.remove_fd(self._fileno, self._read_cb)
                except Exception as e:
                    logging.error("socket close remove_fd error:%s", e)
                self._read_handler = False
            if self._write_handler:
                try:
                    self._loop.remove_fd(self._fileno, self._write_cb)
                except Exception as e:
                    logging.error("socket close remove_fd error:%s", e)
                self._write_handler = False

        self._state = STATE_CLOSED
        def on_close():
            if self._socket:
                try:
                    self._loop.clear_fd(self._fileno)
                except Exception as e:
                    logging.error("server close clear_fd error: %s", e)
                try:
                    self._socket.close()
                except Exception as e:
                    logging.error("socket close socket error:%s", e)

            try:
                self.emit_close(self)
            except Exception as e:
                logging.exception("tcp emit close error:%s", e)
            self.remove_all_listeners()
            self._rbuffers = None
            self._wbuffers = None
        self._loop.add_async(on_close)

    def _error(self, error):
        self._loop.add_async(self.emit_error, self, error)
        self._loop.add_async(self.close)
        logging.error("socket error:%s",error)

    def _connect_cb(self):
        if self._state != STATE_CONNECTING:
            return
        try:
            self._loop.remove_fd(self._fileno, self._connect_cb)
        except Exception as e:
            return self._error(e)
        self._connect_handler = False

        if self._connect_timeout_handler:
            self._loop.cancel_timeout(self._connect_timeout_handler)
            self._connect_timeout_handler = None

        self._state = STATE_STREAMING
        try:
            self._read_handler = self._loop.add_fd(self._fileno, MODE_IN, self._read_cb)
        except Exception as e:
            return self._error(e)

        self._rbuffers.on("drain", lambda _: self.drain())
        self._rbuffers.on("regain", lambda _: self.regain())
        self._loop.add_async(self.emit_connect, self)

        if self._wbuffers and not self._write_handler:
            try:
                self._write_handler = self._loop.add_fd(self._fileno, MODE_OUT, self._write_cb)
            except Exception as e:
                return self._error(e)

    def connect(self, address, timeout=5):
        if self._state != STATE_INITIALIZED:
            return

        self._connect_timeout = timeout

        def on_timeout_cb():
            if self._state == STATE_CONNECTING:
                if not self._is_enable_fast_open:
                    self._error(Exception("connect time out %s" % str(address)))

        def do_connect(hostname, ip):
            if self._state == STATE_CLOSED:
                return

            if not ip:
                return self._loop.add_async(self._error, Exception('can not resolve hostname %s' % str(address)))

            try:
                addrinfo = socket.getaddrinfo(ip, address[1], 0, 0, socket.SOL_TCP)
                if not addrinfo:
                    return  self._loop.add_async(self._error, Exception('can not resolve hostname %s' % str(address)))

                addr = addrinfo[0]
                self._socket = socket.socket(addr[0], addr[1], addr[2])
                self._fileno = self._socket.fileno()
                self._socket_family = addr[0]
                self._socket.setblocking(False)
                self._address = addr[4]
                self._is_resolve = True

                if self._is_enable_fast_open:
                    try:
                        self._socket.setsockopt(socket.SOL_TCP, 23, 5)
                    except Exception as e:
                        logging.warning('fast open error: %s', e)
                        self._is_enable_fast_open = False

                if self._is_enable_nodelay:
                    try:
                        self._socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
                    except Exception as e:
                        logging.warning('nodela error: %s', e)
                        self._is_enable_nodelay = False

                if self._is_enable_fast_open:
                    if self._wbuffers and not self._connect_handler:
                        self._loop.add_async(self._connect_and_write)
                else:
                    self._connect_handler = self._loop.add_fd(self._fileno, MODE_OUT, self._connect_cb)
                    self._socket.connect(self._address)
            except socket.error as e:
                if e.args[0] not in (errno.EINPROGRESS, errno.EWOULDBLOCK):
                    return self._loop.add_async(self._error, Exception("connect error %s %s" % (str(address), e)))
            except Exception as e:
                return self._loop.add_async(self._error, e)

        self._dns_resolver.resolve(address[0], do_connect)
        self._connect_timeout_handler = self._loop.add_timeout(timeout, on_timeout_cb)
        self._state = STATE_CONNECTING

    def drain(self):
        if self._state in (STATE_STREAMING, STATE_CLOSING):
            if self._read_handler:
                try:
                    self._loop.remove_fd(self._fileno, self._read_cb)
                except Exception as e:
                    return self._error(e)
                self._read_handler = False

    def regain(self):
        if self._state in (STATE_STREAMING, STATE_CLOSING):
            if not self._read_handler:
                try:
                    self._read_handler = self._loop.add_fd(self._fileno, MODE_IN, self._read_cb)
                except Exception as e:
                    self._error(e)

    def _read_cb(self):
        if self._state == STATE_CONNECTING:
            if self._read():
                self._connect_cb()
                self._loop.add_async(self.emit_data, self, self._rbuffers)
            else:
                if self._rbuffers._len:
                    self._connect_cb()
                    self._loop.add_async(self.emit_data, self, self._rbuffers)
                self._loop.add_async(self.emit_end, self)
                self.close()
            return

        if self._read():
            self._loop.add_async(self.emit_data, self, self._rbuffers)
        else:
            if self._rbuffers._len:
                self._loop.add_async(self.emit_data, self, self._rbuffers)
            self._loop.add_async(self.emit_end, self)
            if self._state in (STATE_STREAMING, STATE_CLOSING):
                self.close()

    if cbuffer is None:
        def _read(self):
            last_data_len = self._rbuffers._len
            while self._read_handler:
                try:
                    data = self._socket.recv(self.RECV_BUFFER_SIZE)
                    if not data:
                        break
                    self._rbuffers.write(data)
                except socket.error as e:
                    if e.args[0] in (errno.EWOULDBLOCK, errno.EAGAIN):
                        break
                    else:
                        self._error(e)
                        if self._rbuffers._len > self._rbuffers._drain_size and not self._rbuffers._full:
                            self._rbuffers.do_drain()
                        return False
                except Exception as e:
                    self._error(e)
                    return False

            if last_data_len < self._rbuffers._len:
                if self._rbuffers._len > self._rbuffers._drain_size and not self._rbuffers._full:
                    self._rbuffers.do_drain()
                return True
            return False
    else:
        def _read(self):
            try:
                r = self._rbuffers.socket_recv(self._fileno)
            except Exception as e:
                self._error(e)
                if self._rbuffers._len > self._rbuffers._drain_size and not self._rbuffers._full:
                    self._rbuffers.do_drain()
                return False

            if self._rbuffers._len > self._rbuffers._drain_size and not self._rbuffers._full:
                self._rbuffers.do_drain()
            return r

    def _write_cb(self):
        if self._state == STATE_CONNECTING:
            self._connect_cb()
            if self._wbuffers:
                if self._write():
                    if self._has_drain_event:
                        self._loop.add_async(self.emit_drain, self)
                if self._write_handler:
                    try:
                        self._loop.remove_fd(self._fileno, self._write_cb)
                    except Exception as e:
                        return self._error(e)
                    self._write_handler = False
                if self._state == STATE_CLOSING:
                    self.close()
            return

        if self._write():
            if self._has_drain_event:
                self._loop.add_async(self.emit_drain, self)
            if self._write_handler:
                try:
                    self._loop.remove_fd(self._fileno, self._write_cb)
                except Exception as e:
                    return self._error(e)
                self._write_handler = False
            if self._state == STATE_CLOSING:
                self.close()

    def _connect_and_write(self):
        if self._wbuffers:
            def on_timeout_cb():
                if self._state == STATE_CONNECTING:
                    if self._is_enable_fast_open:
                        self._error(Exception("connect time out %s:%s" % (self._address[0], self._address[1])))

            if self._connect_timeout_handler:
                self._loop.cancel_timeout(self._connect_timeout_handler)
                self._connect_timeout_handler = None
            self._connect_timeout_handler = self._loop.add_timeout(self._connect_timeout, on_timeout_cb)

            try:
                self._wbuffers.read(self._socket.sendto(self._wbuffers.join(), MSG_FASTOPEN, self.address))
                self._connect_handler = self._loop.add_fd(self._fileno, MODE_OUT, self._connect_cb)
            except socket.error as e:
                if e.args[0] == errno.EINPROGRESS:
                    try:
                        self._connect_handler = self._loop.add_fd(self._fileno, MODE_OUT, self._connect_cb)
                    except Exception as e:
                        self._error(e)
                    return False
                elif e.args[0] == errno.ENOTCONN:
                    logging.error('fast open not supported on this OS')
                    self._is_enable_fast_open = False
                self._error(e)
                return False
            except Exception as e:
                self._error(e)
                return False

    if cbuffer is None:
        def _write(self):
            while self._wbuffers:
                data = self._wbuffers
                try:
                    if data._buffer_index > 0:
                        r = self._socket.send(memoryview(data._buffer)[data._buffer_index:])
                    else:
                        r = self._socket.send(data._buffer)
                    data._buffer_index += r
                    data._len -= r

                    if data._buffer_index >= data._buffer_len:
                        if data._len > 0:
                            data._buffer = data._buffers.popleft()
                            data._buffer_odata = data._buffers_odata.popleft()
                            data._buffer_index, data._buffer_len = 0, len(data._buffer)
                        else:
                            data._buffer_index, data._buffer_len, data._buffer, data._buffer_odata = 0, 0, b'', None
                            if data._full and data._len < data._regain_size:
                                data.do_regain()
                        continue
                    else:
                        if data._full and data._len < data._regain_size:
                            data.do_regain()
                        return False
                except socket.error as e:
                    if e.args[0] not in (errno.EWOULDBLOCK, errno.EAGAIN):
                        self._error(e)
                    if data._full and data._len < data._regain_size:
                        data.do_regain()
                    return False
                except Exception as e:
                    self._error(e)
                    return False
            return True
    else:
        def _write(self):
            try:
                self._wbuffers.socket_send(self._fileno)
                if self._wbuffers:
                    if self._wbuffers._full and self._wbuffers._len < self._wbuffers._regain_size:
                        self._wbuffers.do_regain()
                    return False
            except Exception as e:
                self._error(e)
                if self._wbuffers._full and self._wbuffers._len < self._wbuffers._regain_size:
                    self._wbuffers.do_regain()
                return False

            if self._wbuffers._full and self._wbuffers._len < self._wbuffers._regain_size:
                self._wbuffers.do_regain()
            return True

    def write(self, data):
        if self._state != STATE_STREAMING:
            if self._state == STATE_CONNECTING:
                if data.__class__ == Buffer:
                    if self._wbuffers != data:
                        if not self._wbuffers:
                            self._wbuffers = data
                        else:
                            while data:
                                self._wbuffers.write(data.next())
                else:
                    if self._wbuffers is None:
                        self._wbuffers = Buffer(max_buffer_size=self._max_buffer_size)
                    self._wbuffers.write(data)

                if self._is_enable_fast_open and self._is_resolve and not self._connect_handler:
                    return self._connect_and_write()
                return False
            assert self._state == STATE_STREAMING, "not connected"

        if data.__class__ == Buffer:
            if self._wbuffers != data:
                if not self._wbuffers:
                    self._wbuffers = data
                else:
                    while data:
                        self._wbuffers.write(data.next())
        else:
            if self._wbuffers is None:
                self._wbuffers = Buffer(max_buffer_size = self._max_buffer_size)
            self._wbuffers.write(data)

        if not self._write_handler:
            if self._write():
                if self._has_drain_event:
                    self._loop.add_async(self.emit_drain, self)
                return True
            try:
                self._write_handler = self._loop.add_fd(self._fileno, MODE_OUT, self._write_cb)
            except Exception as e:
                self._error(e)
        return False


class Socket6(Socket):
    pass


class Server(event.EventEmitter):
    def __init__(self, loop=None, dns_resolver = None):
        super(Server, self).__init__()
        self._loop = loop or instance()
        self._dns_resolver = dns_resolver or DNSResolver.default()
        self._fileno = 0
        self._socket = None
        self._state = STATE_INITIALIZED
        self._accept_handler = False
        self._is_enable_fast_open = False
        self._is_reuseaddr = False
        self._is_enable_nodelay = False
        self._is_resolve = False

    def __del__(self):
        self.close()

    def on_connection(self, callback):
        self.on("connection", callback)

    def on_close(self, callback):
        self.on("close", callback)

    def on_error(self, callback):
        self.on("error", callback)

    def enable_fast_open(self):
        self._is_enable_fast_open = True

    def enable_reuseaddr(self):
        self._is_reuseaddr = True

    def enable_nodelay(self):
        self._is_enable_nodelay = True

    @property
    def is_enable_fast_open(self):
        return self._is_enable_fast_open

    @property
    def is_reuseaddr(self):
        return self._is_reuseaddr

    @property
    def is_enable_nodelay(self):
        return self._is_enable_nodelay

    def listen(self, address, backlog=128):
        if self._state != STATE_INITIALIZED:
            return

        def do_listen(hostname, ip):
            if not ip:
                return self._loop.add_async(self._error, Exception('can not resolve hostname %s' % str(address)))

            try:
                addrinfo = socket.getaddrinfo(ip, address[1], 0, 0, socket.SOL_TCP)
                if not addrinfo:
                    return self._loop.add_async(self._error, Exception('can not resolve hostname %s' % str(address)))

                addr = addrinfo[0]
                self._socket = socket.socket(addr[0], addr[1], addr[2])
                self._fileno = self._socket.fileno()
                self._socket.setblocking(False)
                self._is_resolve = True

                if self._is_reuseaddr:
                    try:
                        self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                    except Exception as e:
                        logging.warning('reuseaddr error: %s', e)
                        self._is_reuseaddr = False

                if self._is_enable_fast_open:
                    try:
                        self._socket.setsockopt(socket.SOL_TCP, 23, 5)
                    except Exception as e:
                        logging.warning('fast open error: %s', e)
                        self._is_enable_fast_open = False

                self._socket.bind(addr[4])
                self._accept_handler = self._loop.add_fd(self._fileno, MODE_IN, self._accept_cb)
                self._socket.listen(backlog)
            except Exception as e:
                self._loop.add_async(self._error, e)

        self._dns_resolver.resolve(address[0], do_listen)
        self._state = STATE_LISTENING

    def _accept_cb(self):
        if self._state != STATE_LISTENING:
            return

        connection, address = self._socket.accept()
        socket = Socket(loop=self._loop, socket=connection, address=address)
        if self._is_enable_nodelay:
            socket.enable_nodelay()
        self._loop.add_async(self.emit_connection, self, socket)

    def _error(self, error):
        self._loop.add_async(self.emit_error, self, error)
        self.close()
        logging.error("server error: %s", error)

    def close(self):
        if self._state in (STATE_INITIALIZED, STATE_LISTENING):
            if self._accept_handler:
                try:
                    self._loop.remove_fd(self._fileno, self._accept_cb)
                except Exception as e:
                    logging.error("server close remove_fd error: %s", e)
                self._accept_handler = False
            if self._socket is not None:
                try:
                    self._loop.clear_fd(self._fileno)
                except Exception as e:
                    logging.error("server close clear_fd error: %s", e)
                try:
                    self._socket.close()
                except Exception as e:
                    logging.error("server close socket error: %s", e)
            self._state = STATE_CLOSED
            def on_close():
                try:
                    self.emit_close(self)
                except Exception as e:
                    logging.exception("tcp server emit close error:%s", e)
                self.remove_all_listeners()
            self._loop.add_async(on_close)


class Server6(Server):
    pass
